'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getNextTextNode = require('./utils/getNextTextNode.js');

var _getNextTextNode2 = _interopRequireDefault(_getNextTextNode);

var _serializeRange = require('./range/serializeRange.js');

var _serializeRange2 = _interopRequireDefault(_serializeRange);

var _highlightRange = require('./range/highlightRange.js');

var _highlightRange2 = _interopRequireDefault(_highlightRange);

var _bodyIndexOf = require('./utils/bodyIndexOf.js');

var _bodyIndexOf2 = _interopRequireDefault(_bodyIndexOf);

var _deserializeRange = require('./range/deserializeRange.js');

var _deserializeRange2 = _interopRequireDefault(_deserializeRange);

var _unfold = require('./utils/unfold.js');

var _unfold2 = _interopRequireDefault(_unfold);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Highlight = function () {
  /**
   * @param {Range} range
   */

  function Highlight(range) {
    _classCallCheck(this, Highlight);

    var sc = range.startContainer;
    var firstTextNode = sc.nodeType === Node.TEXT_NODE ? sc : (0, _getNextTextNode2.default)(sc);

    this.text = range.toString();
    this.index = (0, _bodyIndexOf2.default)(firstTextNode) + range.startOffset;
    this.range = (0, _serializeRange2.default)(range);
    this.nodes = [];

    this.mount();
  }

  /**
   * @returns {boolean}
   */

  _createClass(Highlight, [{
    key: 'mount',
    value: function mount() {
      this.unmount();

      var range = (0, _deserializeRange2.default)(this.range);

      if (range && range.toString() === this.text) {
        this.nodes = (0, _highlightRange2.default)(range);
        document.normalize();
        return true;
      }

      return false;
    }
  }, {
    key: 'unmount',
    value: function unmount() {
      this.nodes.forEach(_unfold2.default);
      document.normalize();
      this.nodes = [];
    }

    /**
     * @param {string} event
     * @param {function} handler
     */

  }, {
    key: 'on',
    value: function on(event, handler) {
      this.nodes.forEach(function (node) {
        node.addEventListener(event, handler);
      });
    }

    /**
     * @param {string} event
     * @param {function} handler
     */

  }, {
    key: 'off',
    value: function off(event, handler) {
      this.nodes.forEach(function (node) {
        node.removeEventListener(event, handler);
      });
    }

    /**
     * @returns {object}
     */

  }, {
    key: 'getBoundingClientRect',
    value: function getBoundingClientRect() {
      var rects = this.nodes.map(function (node) {
        return node.getBoundingClientRect();
      });

      var getBound = function getBound(side, op) {
        return op.apply(undefined, _toConsumableArray(rects.map(function (rect) {
          return rect[side];
        })));
      };

      var top = getBound('top', Math.min);
      var right = getBound('right', Math.max);
      var bottom = getBound('bottom', Math.max);
      var left = getBound('left', Math.min);

      return { top: top, right: right, bottom: bottom, left: left };
    }
  }, {
    key: 'scrollIntoView',
    value: function scrollIntoView() {
      var highlightRect = this.getBoundingClientRect();

      if (highlightRect) {
        var documentRect = document.documentElement.getBoundingClientRect();

        scrollTo(highlightRect.left - documentRect.left, highlightRect.top - documentRect.top);
      }
    }

    /**
     * @param {string} className
     */

  }, {
    key: 'addClass',
    value: function addClass(className) {
      this.nodes.forEach(function (node) {
        node.classList.add(className);
      });
    }

    /**
     * @param {string} className
     */

  }, {
    key: 'removeClass',
    value: function removeClass(className) {
      this.nodes.forEach(function (node) {
        node.classList.remove(className);
      });
    }

    /**
     * @param {string} className
     * @returns {boolean}
     */

  }, {
    key: 'hasClass',
    value: function hasClass(className) {
      return this.nodes.some(function (node) {
        return node.classList.contains(className);
      });
    }

    /**
     * @param {string} className
     * @returns {boolean}
     */

  }, {
    key: 'toggleClass',
    value: function toggleClass(className) {
      this.nodes.forEach(function (node) {
        node.classList.toggle(className);
      });

      return this.hasClass(className);
    }

    /**
     * @param {string} name
     * @returns {boolean}
     */

  }, {
    key: 'hasAttribute',
    value: function hasAttribute(name) {
      return this.nodes.some(function (node) {
        return node.hasAttribute(name);
      });
    }

    /**
     * @param {string} name
     * @returns {?string}
     */

  }, {
    key: 'getAttribute',
    value: function getAttribute(name) {
      if (this.hasAttribute(name)) {
        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = this.nodes[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var node = _step.value;

            if (node.hasAttribute(name)) {
              return node.getAttribute(name);
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }

      return null;
    }

    /**
     * @param {string} name
     * @param {string} [value]
     */

  }, {
    key: 'setAttribute',
    value: function setAttribute(name) {
      var value = arguments.length <= 1 || arguments[1] === undefined ? '' : arguments[1];

      this.nodes.forEach(function (node) {
        node.setAttribute(name, value);
      });
    }

    /**
     * @param {string} name
     */

  }, {
    key: 'removeAttribute',
    value: function removeAttribute(name) {
      this.nodes.forEach(function (node) {
        node.removeAttribute(name);
      });
    }
  }]);

  return Highlight;
}();

exports.default = Highlight;