'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _Highlight = require('./Highlight.js');

var _Highlight2 = _interopRequireDefault(_Highlight);

var _bluebird = require('bluebird');

var _bluebird2 = _interopRequireDefault(_bluebird);

var _Queue = require('./utils/Queue.js');

var _Queue2 = _interopRequireDefault(_Queue);

var _findRange = require('./range/findRange.js');

var _findRange2 = _interopRequireDefault(_findRange);

var _normalFind = require('./utils/normalFind.js');

var _normalFind2 = _interopRequireDefault(_normalFind);

var _deserializeRange = require('./range/deserializeRange.js');

var _deserializeRange2 = _interopRequireDefault(_deserializeRange);

var _fuzzyStringSearch = require('./utils/fuzzyStringSearch.js');

var _fuzzyStringSearch2 = _interopRequireDefault(_fuzzyStringSearch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Highlighter = function () {
  function Highlighter() {
    _classCallCheck(this, Highlighter);

    this.queue = new _Queue2.default();
  }

  _createClass(Highlighter, [{
    key: 'highlightFromRange',

    /**
     * @param {Range} range
     * @returns {Promise}
     */
    value: function highlightFromRange(range) {
      // clone the range before queing in case it changes
      var clone = range.cloneRange();

      return this.queue.add(function () {
        return new _Highlight2.default(clone);
      });
    }

    /**
     * @returns {Promise}
     */

  }, {
    key: 'highlightCurrentSelection',
    value: function highlightCurrentSelection() {
      var _this = this;

      // get the selection before queing in case the user deselects
      var sel = getSelection();

      if (sel.rangeCount > 0) {
        var _ret = function () {
          // clone the range before queing in case it changes
          var range = sel.getRangeAt(0).cloneRange();

          return {
            v: _this.queue.add(function () {
              sel.removeAllRanges();
              return new _Highlight2.default(range);
            })
          };
        }();

        if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
      }

      return new _bluebird2.default(function (resolve) {
        resolve(null);
      });
    }

    /**
     * @param {string} searchValue
     * @param {number} [fromIndex]
     * @returns {Promise}
     */

  }, {
    key: 'highlightFromString',
    value: function highlightFromString(searchValue) {
      var fromIndex = arguments.length <= 1 || arguments[1] === undefined ? 0 : arguments[1];

      return this.queue.add(function () {
        var range = (0, _findRange2.default)(searchValue, fromIndex);

        if (range) {
          return new _Highlight2.default(range);
        }

        // try normalizing all whitespace and searching
        var result = (0, _normalFind2.default)(document.body.textContent, searchValue, fromIndex);
        if (result) {
          range = (0, _findRange2.default)(result.match, result.start);

          if (range) {
            return new _Highlight2.default(range);
          }
        }

        return null;
      });
    }

    /**
     * @param {object} json
     * @param {string} json.range
     * @param {string} json.text
     * @param {number} json.index
     * @returns {Promise}
     */

  }, {
    key: 'highlightFromJson',
    value: function highlightFromJson(json) {
      // clones the object before queing in case it changes
      var clone = JSON.parse(JSON.stringify(json));

      return this.queue.add(function () {
        var range = (0, _deserializeRange2.default)(clone.range);

        if (range && range.toString() === clone.text) {
          return new _Highlight2.default(range);
        }

        var result = (0, _fuzzyStringSearch2.default)(document.body.textContent, clone.text, clone.index);
        if (result) {
          range = (0, _findRange2.default)(result.match, result.start);

          if (range) {
            return new _Highlight2.default(range);
          }
        }

        return null;
      });
    }
  }]);

  return Highlighter;
}();

// Not using es6 `export` because of issues with Browserify standalone options.

module.exports = Highlighter;