'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = evaluateXPath;

var _isTextLike = require('./../utils/isTextLike.js');

var _isTextLike2 = _interopRequireDefault(_isTextLike);

var _isHighlight = require('./../utils/isHighlight.js');

var _isHighlight2 = _interopRequireDefault(_isHighlight);

var _getTextNodeFromOffset = require('./../utils/getTextNodeFromOffset.js');

var _getTextNodeFromOffset2 = _interopRequireDefault(_getTextNodeFromOffset);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @param {string} path
 * @param {number} [offset]
 * @returns {?Node}
 */
function evaluateXPath(path) {
  var offset = arguments.length <= 1 || arguments[1] === undefined ? 0 : arguments[1];

  var idMatch = path.match(/\[@id="(\S+)"\]/);
  var id = idMatch && idMatch[1];
  var p = path;

  // Only evaluate the XPath using the optimized id option if there is only one
  // element with that id in the entire document
  if (id && document.querySelectorAll('#' + id).length > 1) {
    p = p.split('|')[1];
  }

  var regex = /\/text\(\)\[(\d+)\]/;
  var match = p.match(regex);

  // check if the XPath pointed to a text node and if it did get its parent,
  // find the text node using the XPath's index and then find the text node at
  // the given offset
  if (match) {
    var index = parseInt(match[1], 10);

    // if the XPath is invalid or it's not found then an error will be thrown
    var sibling = undefined;
    try {
      sibling = document.evaluate(p.replace(new RegExp(regex.source, 'g'), ''), document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue.firstChild;
    } catch (e) {
      return null;
    }

    // find the XPath's text node
    var i = 0;
    while (sibling) {
      if ((0, _isTextLike2.default)(sibling)) {
        // count the text only if it's not adjacent to a highlight
        if (!(sibling.previousSibling && (0, _isTextLike2.default)(sibling.previousSibling))) {
          i++;
        }

        // check if the text node is found
        if (i === index) {
          // if it's a highlight then the text node is the first descendant text
          // node
          if ((0, _isHighlight2.default)(sibling)) {
            while (sibling.firstChild) {
              sibling = sibling.firstChild;
            }
          }
          return (0, _getTextNodeFromOffset2.default)(sibling, offset, true);
        }
      }

      sibling = sibling.nextSibling;
    }
  } else {
    // if the XPath is invalid or it's not found then an error will be thrown
    try {
      return document.evaluate(p, document, null, XPathResult.FIRST_ORDERED_NODE_TYPE, null).singleNodeValue;
    } catch (e) {
      return null;
    }
  }

  return null;
}